use actions_proto_rs::ping_service_client::PingServiceClient;
use actions_proto_rs::runner_service_client::RunnerServiceClient;
use actions_proto_rs::RegisterRequest;
use hyper::client::HttpConnector;
use hyper::Client;
use hyper_tls::HttpsConnector;
use tonic::codegen::InterceptedService;
use tonic::metadata::AsciiMetadataValue;
use tonic::metadata::MetadataMap;
use tonic::metadata::MetadataValue;
use tonic::service::Interceptor;
use tonic::Extensions;
use tonic::{transport::Channel, Request};
use tonic_web::GrpcWebClientLayer;

const UUID_HEADER_KEY: &str = "x-runner-uuid";
const TOKEN_HEADER_KEY: &str = "x-runner-token";
const VERSION_HEADER_KEY: &str = "x-runner-version";

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let token = "V4jpnTdZJBOVawC7tuQqZwdPpAnORlygg8Uoz2ff";
    // version needcs to be in v3.4.1 format
    // so concat v
    let version = concat!("v", env!("CARGO_PKG_VERSION"));
    let version = "v3.4.1";
    let uuid = uuid::Uuid::new_v4().to_string();

    let client = hyper::Client::builder().build(HttpsConnector::new());

    let svc = tower::ServiceBuilder::new()
        .layer(GrpcWebClientLayer::new())
        .service(client);

    let mut runner_client = RunnerServiceClient::with_origin(
        svc.clone(),
        "https://git.ok.software/api/actions".try_into()?,
    );
    let mut ping_client =
        PingServiceClient::with_origin(svc, "https://git.ok.software/api/actions".try_into()?);

    // Log the client creation
    println!("Client created with UUID: {}", uuid);
    let mut metadata = MetadataMap::new();
    metadata.insert(VERSION_HEADER_KEY, version.parse().unwrap());

    metadata.insert(UUID_HEADER_KEY, uuid.parse().unwrap());
    metadata.insert(TOKEN_HEADER_KEY, token.parse().unwrap());
    metadata.insert(VERSION_HEADER_KEY, version.parse().unwrap());

    // register meta data needs to be empty
    let reg_metadata = metadata.clone();
    // ping request
    let ping_request = Request::from_parts(
        reg_metadata.clone(),
        Extensions::default(),
        actions_proto_rs::PingRequest {
            data: "nixos".to_string(),
        },
    );

    // Log ping request
    println!("Ping Request: {:?}", ping_request);
    match ping_client.ping(ping_request).await {
        Ok(resp) => {
            println!("Response: {:?}", resp);
        }
        Err(e) => {
            println!("Error: {:?}", e);
        }
    }

    let labels = vec![
        "ubuntu-latest:docker://gitea/runner-images:ubuntu-latest".to_string(),
        "ubuntu-22.04:docker://gitea/runner-images:ubuntu-22.04".to_string(),
        "ubuntu-20.04:docker://gitea/runner-images:ubuntu-20.04".to_string(),
    ];

    // Log metadata
    println!("Metadata: {:?}", metadata);
    let request = Request::from_parts(
        reg_metadata,
        Extensions::default(),
        RegisterRequest {
            name: "nixos".to_string(),
            version: version.to_string(),
            token: token.to_string(),
            labels: labels.clone(),
            agent_labels: labels,
            custom_labels: vec![],
        },
    );

    // Log request
    println!("Request: {:?}", request);
    let resp = runner_client.register(request).await;
    match resp {
        Ok(resp) => {
            println!("Response: {:?}", resp);
        }
        Err(e) => {
            println!("Error: {:?}", e);
        }
    }

    let fetch_task_request = Request::from_parts(
        metadata,
        Extensions::default(),
        actions_proto_rs::FetchTaskRequest { tasks_version: 0 },
    );

    // Log fetch task request
    println!("Fetch Task Request: {:?}", fetch_task_request);
    let response = runner_client.fetch_task(fetch_task_request).await?;

    Ok(())
}
